/* automatically generated from xfpm-settings.ui */
#ifdef __SUNPRO_C
#pragma align 4 (xfpm_settings_ui)
#endif
#ifdef __GNUC__
static const char xfpm_settings_ui[] __attribute__ ((__aligned__ (4))) =
#else
static const char xfpm_settings_ui[] =
#endif
{
  "<?xml version=\"1.0\" encoding=\"UTF-8\"?><interface><requires lib=\"gt"
  "k+\" version=\"3.20\"/><requires lib=\"libxfce4ui-2\" version=\"4.12\"/"
  "><object class=\"GtkAdjustment\" id=\"brightness-inactivity-on-ac-adjus"
  "tment\"><property name=\"lower\">9</property><property name=\"upper\">1"
  "20</property><property name=\"value\">9</property><property name=\"step"
  "_increment\">1</property><property name=\"page_increment\">10</property"
  "></object><object class=\"GtkAdjustment\" id=\"brightness-inactivity-on"
  "-battery-adjustment\"><property name=\"lower\">9</property><property na"
  "me=\"upper\">120</property><property name=\"value\">30</property><prope"
  "rty name=\"step_increment\">1</property><property name=\"page_increment"
  "\">10</property></object><object class=\"GtkAdjustment\" id=\"brightnes"
  "s-level-on-ac-adjustment\"><property name=\"lower\">1</property><proper"
  "ty name=\"upper\">100</property><property name=\"value\">80</property><"
  "property name=\"step_increment\">1</property><property name=\"page_incr"
  "ement\">10</property></object><object class=\"GtkAdjustment\" id=\"brig"
  "htness-level-on-battery-adjustment\"><property name=\"lower\">1</proper"
  "ty><property name=\"upper\">100</property><property name=\"value\">20</"
  "property><property name=\"step_increment\">1</property><property name=\""
  "page_increment\">10</property></object><object class=\"GtkAdjustment\" "
  "id=\"brightness-step-count-spin-adjustment\"><property name=\"lower\">2"
  "</property><property name=\"upper\">100</property><property name=\"valu"
  "e\">10</property><property name=\"step_increment\">5</property></object"
  "><object class=\"GtkAdjustment\" id=\"critical-power-level-spin-adjustm"
  "ent\"><property name=\"lower\">1</property><property name=\"upper\">70<"
  "/property><property name=\"value\">5</property><property name=\"step_in"
  "crement\">1</property><property name=\"page_increment\">10</property></"
  "object><object class=\"GtkAdjustment\" id=\"display-blank-on-ac-adjustm"
  "ent\"><property name=\"upper\">60</property><property name=\"value\">10"
  "</property><property name=\"step_increment\">1</property><property name"
  "=\"page_increment\">5</property></object><object class=\"GtkAdjustment\""
  " id=\"display-blank-on-battery-adjustment\"><property name=\"upper\">60"
  "</property><property name=\"value\">1</property><property name=\"step_i"
  "ncrement\">1</property><property name=\"page_increment\">5</property></"
  "object><object class=\"GtkAdjustment\" id=\"dpms-off-on-ac-adjustment\""
  "><property name=\"upper\">60</property><property name=\"value\">20</pro"
  "perty><property name=\"step_increment\">1</property><property name=\"pa"
  "ge_increment\">5</property></object><object class=\"GtkAdjustment\" id="
  "\"dpms-off-on-battery-adjustment\"><property name=\"upper\">60</propert"
  "y><property name=\"value\">5</property><property name=\"step_increment\""
  ">1</property><property name=\"page_increment\">5</property></object><ob"
  "ject class=\"GtkAdjustment\" id=\"dpms-sleep-on-ac-adjustment\"><proper"
  "ty name=\"upper\">60</property><property name=\"value\">15</property><p"
  "roperty name=\"step_increment\">1</property><property name=\"page_incre"
  "ment\">5</property></object><object class=\"GtkAdjustment\" id=\"dpms-s"
  "leep-on-battery-adjustment\"><property name=\"upper\">60</property><pro"
  "perty name=\"value\">3</property><property name=\"step_increment\">1</p"
  "roperty><property name=\"page_increment\">5</property></object><object "
  "class=\"GtkImage\" id=\"image1\"><property name=\"visible\">True</prope"
  "rty><property name=\"can_focus\">False</property><property name=\"icon_"
  "name\">help-browser</property></object><object class=\"GtkImage\" id=\""
  "image2\"><property name=\"visible\">True</property><property name=\"can"
  "_focus\">False</property><property name=\"icon_name\">window-close-symb"
  "olic</property></object><object class=\"GtkAdjustment\" id=\"light-lock"
  "er-late-locking-adjustment\"><property name=\"lower\">1</property><prop"
  "erty name=\"upper\">120</property><property name=\"value\">1</property>"
  "<property name=\"step_increment\">1</property><property name=\"page_inc"
  "rement\">10</property></object><object class=\"GtkListStore\" id=\"list"
  "store1\"><columns><column type=\"gint\"/><column type=\"gchararray\"/><"
  "/columns><data><row><col id=\"0\">0</col><col id=\"1\" translatable=\"y"
  "es\">Never</col></row><row><col id=\"0\">1</col><col id=\"1\" translata"
  "ble=\"yes\">When the screensaver is activated</col></row><row><col id=\""
  "0\">2</col><col id=\"1\" translatable=\"yes\">When the screensaver is d"
  "eactivated</col></row></data></object><object class=\"GtkListStore\" id"
  "=\"model1\"><columns><column type=\"gchararray\"/></columns><data><row>"
  "<col id=\"0\" translatable=\"yes\">Nothing</col></row></data></object><"
  "object class=\"GtkListStore\" id=\"model10\"><columns><column type=\"gc"
  "hararray\"/></columns><data><row><col id=\"0\" translatable=\"yes\">Not"
  "hing</col></row></data></object><object class=\"GtkListStore\" id=\"mod"
  "el11\"><columns><column type=\"gchararray\"/></columns><data><row><col "
  "id=\"0\" translatable=\"yes\">Nothing</col></row></data></object><objec"
  "t class=\"GtkListStore\" id=\"model3\"><columns><column type=\"gchararr"
  "ay\"/></columns><data><row><col id=\"0\" translatable=\"yes\">Nothing</"
  "col></row></data></object><object class=\"GtkListStore\" id=\"model4\">"
  "<columns><column type=\"gchararray\"/></columns><data><row><col id=\"0\""
  " translatable=\"yes\">Nothing</col></row></data></object><object class="
  "\"GtkListStore\" id=\"model5\"><columns><column type=\"gchararray\"/></"
  "columns><data><row><col id=\"0\" translatable=\"yes\">Nothing</col></ro"
  "w></data></object><object class=\"GtkListStore\" id=\"model6\"><columns"
  "><column type=\"gchararray\"/></columns><data><row><col id=\"0\" transl"
  "atable=\"yes\">Nothing</col></row></data></object><object class=\"GtkLi"
  "stStore\" id=\"model7\"><columns><column type=\"gchararray\"/></columns"
  "><data><row><col id=\"0\" translatable=\"yes\">Nothing</col></row></dat"
  "a></object><object class=\"GtkListStore\" id=\"model8\"><columns><colum"
  "n type=\"gchararray\"/></columns><data><row><col id=\"0\" translatable="
  "\"yes\">Nothing</col></row></data></object><object class=\"GtkListStore"
  "\" id=\"model9\"><columns><column type=\"gchararray\"/></columns><data>"
  "<row><col id=\"0\" translatable=\"yes\">Nothing</col></row></data></obj"
  "ect><object class=\"GtkAdjustment\" id=\"system-sleep-inactivity-on-ac-"
  "adjustment\"><property name=\"lower\">14</property><property name=\"upp"
  "er\">360</property><property name=\"value\">14</property><property name"
  "=\"step_increment\">1</property><property name=\"page_increment\">10</p"
  "roperty><property name=\"page_size\">10</property></object><object clas"
  "s=\"GtkAdjustment\" id=\"system-sleep-inactivity-on-battery-adjustment\""
  "><property name=\"lower\">14</property><property name=\"upper\">360</pr"
  "operty><property name=\"value\">15</property><property name=\"step_incr"
  "ement\">1</property><property name=\"page_increment\">10</property><pro"
  "perty name=\"page_size\">10</property></object><object class=\"XfceTitl"
  "edDialog\" id=\"xfpm-settings-dialog\"><property name=\"can_focus\">Fal"
  "se</property><property name=\"title\" translatable=\"yes\">Power Manage"
  "r</property><property name=\"window_position\">center-on-parent</proper"
  "ty><property name=\"icon_name\">org.xfce.powermanager</property><proper"
  "ty name=\"type_hint\">normal</property><child internal-child=\"vbox\"><"
  "object class=\"GtkBox\" id=\"dialog-vbox\"><property name=\"visible\">T"
  "rue</property><property name=\"can_focus\">False</property><property na"
  "me=\"orientation\">vertical</property><child internal-child=\"action_ar"
  "ea\"><object class=\"GtkButtonBox\" id=\"dialog-action_area\"><property"
  " name=\"visible\">True</property><property name=\"can_focus\">False</pr"
  "operty><child><object class=\"GtkButton\" id=\"help-button\"><property "
  "name=\"label\" translatable=\"yes\">_Help</property><property name=\"vi"
  "sible\">True</property><property name=\"can_focus\">True</property><pro"
  "perty name=\"receives_default\">True</property><property name=\"image\""
  ">image1</property><property name=\"use_underline\">True</property></obj"
  "ect><packing><property name=\"expand\">False</property><property name=\""
  "fill\">False</property><property name=\"position\">0</property></packin"
  "g></child><child><object class=\"GtkButton\" id=\"close-button\"><prope"
  "rty name=\"label\" translatable=\"yes\">_Close</property><property name"
  "=\"visible\">True</property><property name=\"can_focus\">True</property"
  "><property name=\"receives_default\">True</property><property name=\"im"
  "age\">image2</property><property name=\"use_underline\">True</property>"
  "</object><packing><property name=\"expand\">False</property><property n"
  "ame=\"fill\">False</property><property name=\"position\">1</property></"
  "packing></child></object><packing><property name=\"expand\">False</prop"
  "erty><property name=\"fill\">True</property><property name=\"position\""
  ">0</property></packing></child><child><object class=\"GtkBox\" id=\"plu"
  "g-child\"><property name=\"visible\">True</property><property name=\"ca"
  "n_focus\">False</property><property name=\"border_width\">6</property><"
  "property name=\"orientation\">vertical</property><child><object class=\""
  "GtkNotebook\" id=\"main-notebook\"><property name=\"visible\">True</pro"
  "perty><property name=\"can_focus\">True</property><child><object class="
  "\"GtkBox\" id=\"general-tab\"><property name=\"visible\">True</property"
  "><property name=\"can_focus\">False</property><property name=\"border_w"
  "idth\">12</property><property name=\"orientation\">vertical</property><"
  "property name=\"spacing\">6</property><child><object class=\"GtkFrame\""
  " id=\"buttons-frame\"><property name=\"visible\">True</property><proper"
  "ty name=\"can_focus\">False</property><property name=\"label_xalign\">0"
  "</property><property name=\"shadow_type\">none</property><child><object"
  " class=\"GtkBox\" id=\"vbox2\"><property name=\"visible\">True</propert"
  "y><property name=\"can_focus\">False</property><property name=\"margin_"
  "start\">18</property><property name=\"margin_top\">6</property><propert"
  "y name=\"margin_bottom\">18</property><property name=\"orientation\">ve"
  "rtical</property><property name=\"spacing\">5</property><child><object "
  "class=\"GtkGrid\" id=\"table1\"><property name=\"visible\">True</proper"
  "ty><property name=\"can_focus\">False</property><property name=\"row_sp"
  "acing\">6</property><property name=\"column_spacing\">6</property><chil"
  "d><object class=\"GtkLabel\" id=\"button-power-label\"><property name=\""
  "visible\">True</property><property name=\"can_focus\">False</property><"
  "property name=\"label\" translatable=\"yes\">When power button is press"
  "ed:</property><property name=\"xalign\">0</property></object><packing><"
  "property name=\"left_attach\">0</property><property name=\"top_attach\""
  ">0</property></packing></child><child><object class=\"GtkLabel\" id=\"b"
  "utton-sleep-label\"><property name=\"visible\">True</property><property"
  " name=\"can_focus\">False</property><property name=\"label\" translatab"
  "le=\"yes\">When sleep button is pressed:</property><property name=\"xal"
  "ign\">0</property></object><packing><property name=\"left_attach\">0</p"
  "roperty><property name=\"top_attach\">1</property></packing></child><ch"
  "ild><object class=\"GtkLabel\" id=\"button-hibernate-label\"><property "
  "name=\"visible\">True</property><property name=\"can_focus\">False</pro"
  "perty><property name=\"label\" translatable=\"yes\">When hibernate butt"
  "on is pressed:</property><property name=\"xalign\">0</property></object"
  "><packing><property name=\"left_attach\">0</property><property name=\"t"
  "op_attach\">2</property></packing></child><child><object class=\"GtkLab"
  "el\" id=\"button-battery-label\"><property name=\"visible\">True</prope"
  "rty><property name=\"can_focus\">False</property><property name=\"label"
  "\" translatable=\"yes\">When battery button is pressed:</property><prop"
  "erty name=\"xalign\">0</property></object><packing><property name=\"lef"
  "t_attach\">0</property><property name=\"top_attach\">3</property></pack"
  "ing></child><child><object class=\"GtkComboBox\" id=\"button-power-comb"
  "o\"><property name=\"visible\">True</property><property name=\"can_focu"
  "s\">False</property><property name=\"hexpand\">True</property><signal n"
  "ame=\"changed\" handler=\"button_power_changed_cb\" swapped=\"no\"/><ch"
  "ild><object class=\"GtkCellRendererText\" id=\"renderer1\"/><attributes"
  "><attribute name=\"text\">0</attribute></attributes></child></object><p"
  "acking><property name=\"left_attach\">1</property><property name=\"top_"
  "attach\">0</property></packing></child><child><object class=\"GtkComboB"
  "ox\" id=\"button-sleep-combo\"><property name=\"visible\">True</propert"
  "y><property name=\"can_focus\">False</property><property name=\"hexpand"
  "\">True</property><signal name=\"changed\" handler=\"button_sleep_chang"
  "ed_cb\" swapped=\"no\"/><child><object class=\"GtkCellRendererText\" id"
  "=\"renderer4\"/><attributes><attribute name=\"text\">0</attribute></att"
  "ributes></child></object><packing><property name=\"left_attach\">1</pro"
  "perty><property name=\"top_attach\">1</property></packing></child><chil"
  "d><object class=\"GtkComboBox\" id=\"button-hibernate-combo\"><property"
  " name=\"visible\">True</property><property name=\"can_focus\">False</pr"
  "operty><property name=\"hexpand\">True</property><signal name=\"changed"
  "\" handler=\"button_hibernate_changed_cb\" swapped=\"no\"/><child><obje"
  "ct class=\"GtkCellRendererText\" id=\"renderer3\"/><attributes><attribu"
  "te name=\"text\">0</attribute></attributes></child></object><packing><p"
  "roperty name=\"left_attach\">1</property><property name=\"top_attach\">"
  "2</property></packing></child><child><object class=\"GtkComboBox\" id=\""
  "button-battery-combo\"><property name=\"visible\">True</property><prope"
  "rty name=\"can_focus\">False</property><property name=\"hexpand\">True<"
  "/property><signal name=\"changed\" handler=\"button_battery_changed_cb\""
  " swapped=\"no\"/><child><object class=\"GtkCellRendererText\" id=\"rend"
  "erer5\"/><attributes><attribute name=\"text\">0</attribute></attributes"
  "></child></object><packing><property name=\"left_attach\">1</property><"
  "property name=\"top_attach\">3</property></packing></child><child><obje"
  "ct class=\"GtkSwitch\" id=\"handle-brightness-keys\"><property name=\"v"
  "isible\">True</property><property name=\"can_focus\">True</property><pr"
  "operty name=\"halign\">end</property><signal name=\"state-set\" handler"
  "=\"handle_brightness_keys_toggled_cb\" swapped=\"no\"/></object><packin"
  "g><property name=\"left_attach\">1</property><property name=\"top_attac"
  "h\">4</property></packing></child><child><object class=\"GtkBox\" id=\""
  "hbox4\"><property name=\"visible\">True</property><property name=\"can_"
  "focus\">False</property><property name=\"spacing\">6</property><child><"
  "object class=\"GtkSpinButton\" id=\"brightness-step-count-spin\"><prope"
  "rty name=\"visible\">True</property><property name=\"can_focus\">True</"
  "property><property name=\"hexpand\">False</property><property name=\"in"
  "visible_char\">\342\227\217</property><property name=\"primary_icon_act"
  "ivatable\">False</property><property name=\"secondary_icon_activatable\""
  ">False</property><property name=\"adjustment\">brightness-step-count-sp"
  "in-adjustment</property><signal name=\"value-changed\" handler=\"bright"
  "ness_step_count_value_changed_cb\" swapped=\"no\"/></object><packing><p"
  "roperty name=\"expand\">False</property><property name=\"fill\">False</"
  "property></packing></child><child><object class=\"GtkCheckButton\" id=\""
  "brightness-exponential\"><property name=\"label\" translatable=\"yes\">"
  "Exponential</property><property name=\"visible\">True</property><proper"
  "ty name=\"can_focus\">True</property><property name=\"receives_default\""
  ">False</property><property name=\"draw_indicator\">True</property><sign"
  "al name=\"toggled\" handler=\"brightness_exponential_toggled_cb\" swapp"
  "ed=\"no\"/></object><packing><property name=\"expand\">False</property>"
  "<property name=\"fill\">True</property></packing></child></object><pack"
  "ing><property name=\"left_attach\">1</property><property name=\"top_att"
  "ach\">5</property></packing></child><child><object class=\"GtkLabel\" i"
  "d=\"brightness-step-count-label\"><property name=\"visible\">True</prop"
  "erty><property name=\"can_focus\">False</property><property name=\"xali"
  "gn\">0</property><property name=\"label\" translatable=\"yes\">Brightne"
  "ss step count:</property></object><packing><property name=\"left_attach"
  "\">0</property><property name=\"top_attach\">5</property></packing></ch"
  "ild><child><object class=\"GtkLabel\" id=\"handle-brightness-keys-label"
  "\"><property name=\"visible\">True</property><property name=\"can_focus"
  "\">False</property><property name=\"label\" translatable=\"yes\">Handle"
  " display brightness _keys</property><property name=\"use_underline\">Tr"
  "ue</property><property name=\"ellipsize\">end</property><property name="
  "\"xalign\">0</property></object><packing><property name=\"left_attach\""
  ">0</property><property name=\"top_attach\">4</property></packing></chil"
  "d></object><packing><property name=\"expand\">True</property><property "
  "name=\"fill\">True</property><property name=\"position\">0</property></"
  "packing></child></object></child><child type=\"label\"><object class=\""
  "GtkLabel\" id=\"label1\"><property name=\"visible\">True</property><pro"
  "perty name=\"can_focus\">False</property><property name=\"label\" trans"
  "latable=\"yes\">&lt;b&gt;Buttons&lt;/b&gt;</property><property name=\"u"
  "se_markup\">True</property></object></child></object><packing><property"
  " name=\"expand\">False</property><property name=\"fill\">True</property"
  "><property name=\"position\">0</property></packing></child><child><obje"
  "ct class=\"GtkFrame\" id=\"appearance-frame\"><property name=\"visible\""
  ">True</property><property name=\"can_focus\">False</property><property "
  "name=\"label_xalign\">0</property><property name=\"shadow_type\">none</"
  "property><child><object class=\"GtkBox\" id=\"vbox10\"><property name=\""
  "visible\">True</property><property name=\"can_focus\">False</property><"
  "property name=\"margin_start\">18</property><property name=\"margin_top"
  "\">6</property><property name=\"margin_bottom\">18</property><property "
  "name=\"orientation\">vertical</property><property name=\"spacing\">6</p"
  "roperty><child><object class=\"GtkBox\" id=\"hbox20\"><property name=\""
  "visible\">True</property><property name=\"can_focus\">False</property><"
  "child><object class=\"GtkLabel\" id=\"show-notifications-label\"><prope"
  "rty name=\"visible\">True</property><property name=\"can_focus\">False<"
  "/property><property name=\"label\" translatable=\"yes\">Status notifica"
  "tions</property><property name=\"use_underline\">True</property><proper"
  "ty name=\"ellipsize\">end</property><property name=\"xalign\">0</proper"
  "ty></object><packing><property name=\"expand\">True</property><property"
  " name=\"fill\">True</property><property name=\"position\">0</property><"
  "/packing></child><child><object class=\"GtkSwitch\" id=\"show-notificat"
  "ions\"><property name=\"visible\">True</property><property name=\"can_f"
  "ocus\">True</property><property name=\"halign\">end</property></object>"
  "<packing><property name=\"expand\">False</property><property name=\"fil"
  "l\">True</property><property name=\"position\">1</property></packing></"
  "child></object><packing><property name=\"expand\">True</property><prope"
  "rty name=\"fill\">True</property><property name=\"position\">1</propert"
  "y></packing></child><child><object class=\"GtkBox\" id=\"hbox21\"><prop"
  "erty name=\"visible\">True</property><property name=\"can_focus\">False"
  "</property><child><object class=\"GtkLabel\" id=\"show-systray-label\">"
  "<property name=\"visible\">True</property><property name=\"can_focus\">"
  "False</property><property name=\"label\" translatable=\"yes\">System tr"
  "ay icon</property><property name=\"use_underline\">True</property><prop"
  "erty name=\"ellipsize\">end</property><property name=\"xalign\">0</prop"
  "erty></object><packing><property name=\"expand\">True</property><proper"
  "ty name=\"fill\">True</property><property name=\"position\">0</property"
  "></packing></child><child><object class=\"GtkSwitch\" id=\"show-systray"
  "\"><property name=\"visible\">True</property><property name=\"can_focus"
  "\">True</property><property name=\"halign\">end</property></object><pac"
  "king><property name=\"expand\">False</property><property name=\"fill\">"
  "True</property><property name=\"position\">1</property></packing></chil"
  "d></object><packing><property name=\"expand\">True</property><property "
  "name=\"fill\">True</property><property name=\"position\">2</property></"
  "packing></child></object></child><child type=\"label\"><object class=\""
  "GtkLabel\" id=\"label2\"><property name=\"visible\">True</property><pro"
  "perty name=\"can_focus\">False</property><property name=\"label\" trans"
  "latable=\"yes\">&lt;b&gt;Appearance&lt;/b&gt;</property><property name="
  "\"use_markup\">True</property></object></child></object><packing><prope"
  "rty name=\"expand\">False</property><property name=\"fill\">True</prope"
  "rty><property name=\"position\">2</property></packing></child></object>"
  "<packing><property name=\"position\">1</property></packing></child><chi"
  "ld type=\"tab\"><object class=\"GtkLabel\" id=\"general\"><property nam"
  "e=\"visible\">True</property><property name=\"can_focus\">False</proper"
  "ty><property name=\"label\" translatable=\"yes\">General</property></ob"
  "ject><packing><property name=\"position\">1</property><property name=\""
  "tab_fill\">False</property></packing></child><child><object class=\"Gtk"
  "Box\" id=\"system-tab\"><property name=\"visible\">True</property><prop"
  "erty name=\"can_focus\">False</property><property name=\"border_width\""
  ">12</property><property name=\"orientation\">vertical</property><proper"
  "ty name=\"spacing\">6</property><child><object class=\"GtkStackSwitcher"
  "\" id=\"system-stack-switcher\"><property name=\"visible\">True</proper"
  "ty><property name=\"can_focus\">False</property><property name=\"halign"
  "\">center</property><property name=\"valign\">center</property><propert"
  "y name=\"margin_bottom\">6</property><property name=\"stack\">system-st"
  "ack</property></object><packing><property name=\"expand\">False</proper"
  "ty><property name=\"fill\">True</property><property name=\"position\">0"
  "</property></packing></child><child><object class=\"GtkStack\" id=\"sys"
  "tem-stack\"><property name=\"visible\">True</property><property name=\""
  "can_focus\">False</property><property name=\"transition_type\">slide-le"
  "ft-right</property><child><object class=\"GtkGrid\"><property name=\"vi"
  "sible\">True</property><property name=\"can_focus\">False</property><pr"
  "operty name=\"row_spacing\">6</property><property name=\"column_spacing"
  "\">12</property><child><object class=\"GtkLabel\" id=\"system-sleep-ina"
  "ctivity-label\"><property name=\"visible\">True</property><property nam"
  "e=\"can_focus\">False</property><property name=\"margin_left\">18</prop"
  "erty><property name=\"label\" translatable=\"yes\">When inactive for</p"
  "roperty><property name=\"xalign\">0</property></object><packing><proper"
  "ty name=\"left_attach\">0</property><property name=\"top_attach\">2</pr"
  "operty></packing></child><child><object class=\"GtkBox\"><property name"
  "=\"visible\">True</property><property name=\"can_focus\">False</propert"
  "y><property name=\"orientation\">vertical</property><child><object clas"
  "s=\"GtkScale\" id=\"system-sleep-inactivity-on-battery\"><property name"
  "=\"visible\">True</property><property name=\"can_focus\">True</property"
  "><property name=\"adjustment\">system-sleep-inactivity-on-battery-adjus"
  "tment</property><property name=\"round_digits\">0</property><property n"
  "ame=\"digits\">0</property><property name=\"draw_value\">False</propert"
  "y><signal name=\"value-changed\" handler=\"inactivity_on_battery_value_"
  "changed_cb\" swapped=\"no\"/><style><class name=\"xfce4-scale-label\"/>"
  "</style></object><packing><property name=\"expand\">False</property><pr"
  "operty name=\"fill\">True</property><property name=\"position\">0</prop"
  "erty></packing></child><child><object class=\"GtkLabel\" id=\"system-sl"
  "eep-inactivity-on-battery-label\"><property name=\"visible\">True</prop"
  "erty><property name=\"sensitive\">False</property><property name=\"can_"
  "focus\">False</property><property name=\"xalign\">0.5</property></objec"
  "t><packing><property name=\"expand\">False</property><property name=\"f"
  "ill\">True</property><property name=\"position\">1</property></packing>"
  "</child></object><packing><property name=\"left_attach\">1</property><p"
  "roperty name=\"top_attach\">2</property></packing></child><child><objec"
  "t class=\"GtkComboBox\" id=\"system-sleep-mode-on-battery\"><property n"
  "ame=\"visible\">True</property><property name=\"can_focus\">False</prop"
  "erty><property name=\"hexpand\">True</property><signal name=\"changed\""
  " handler=\"on_battery_sleep_mode_changed_cb\" swapped=\"no\"/><child><o"
  "bject class=\"GtkCellRendererText\" id=\"renderer8\"/><attributes><attr"
  "ibute name=\"text\">0</attribute></attributes></child></object><packing"
  "><property name=\"left_attach\">1</property><property name=\"top_attach"
  "\">1</property></packing></child><child><object class=\"GtkLabel\" id=\""
  "system-sleep-mode-label\"><property name=\"visible\">True</property><pr"
  "operty name=\"can_focus\">False</property><property name=\"margin_left\""
  ">18</property><property name=\"label\" translatable=\"yes\">System slee"
  "p mode:</property><property name=\"xalign\">0</property></object><packi"
  "ng><property name=\"left_attach\">0</property><property name=\"top_atta"
  "ch\">1</property></packing></child><child><object class=\"GtkLabel\" id"
  "=\"label10\"><property name=\"visible\">True</property><property name=\""
  "can_focus\">False</property><property name=\"label\" translatable=\"yes"
  "\">&lt;b&gt;System power saving&lt;/b&gt;</property><property name=\"us"
  "e_markup\">True</property><property name=\"xalign\">0</property></objec"
  "t><packing><property name=\"left_attach\">0</property><property name=\""
  "top_attach\">0</property><property name=\"width\">2</property></packing"
  "></child><child><object class=\"GtkLabel\" id=\"lid-on-battery-header\""
  "><property name=\"visible\">True</property><property name=\"can_focus\""
  ">False</property><property name=\"margin_top\">12</property><property n"
  "ame=\"label\" translatable=\"yes\">&lt;b&gt;Laptop Lid&lt;/b&gt;</prope"
  "rty><property name=\"use_markup\">True</property><property name=\"xalig"
  "n\">0</property></object><packing><property name=\"left_attach\">0</pro"
  "perty><property name=\"top_attach\">3</property><property name=\"width\""
  ">2</property></packing></child><child><object class=\"GtkLabel\" id=\"l"
  "id-action-label\"><property name=\"visible\">True</property><property n"
  "ame=\"can_focus\">False</property><property name=\"margin_left\">18</pr"
  "operty><property name=\"label\" translatable=\"yes\">When laptop lid is"
  " closed:</property><property name=\"xalign\">0</property></object><pack"
  "ing><property name=\"left_attach\">0</property><property name=\"top_att"
  "ach\">4</property></packing></child><child><object class=\"GtkComboBox\""
  " id=\"lid-on-battery-combo\"><property name=\"visible\">True</property>"
  "<property name=\"can_focus\">False</property><property name=\"hexpand\""
  ">True</property><signal name=\"changed\" handler=\"on_battery_lid_chang"
  "ed_cb\" swapped=\"no\"/><child><object class=\"GtkCellRendererText\" id"
  "=\"renderer9\"/><attributes><attribute name=\"text\">0</attribute></att"
  "ributes></child></object><packing><property name=\"left_attach\">1</pro"
  "perty><property name=\"top_attach\">4</property></packing></child></obj"
  "ect><packing><property name=\"name\">page0</property><property name=\"t"
  "itle\" translatable=\"yes\">On battery</property></packing></child><chi"
  "ld><object class=\"GtkGrid\"><property name=\"visible\">True</property>"
  "<property name=\"can_focus\">False</property><property name=\"row_spaci"
  "ng\">6</property><property name=\"column_spacing\">12</property><child>"
  "<object class=\"GtkLabel\" id=\"lid-action-label1\"><property name=\"vi"
  "sible\">True</property><property name=\"can_focus\">False</property><pr"
  "operty name=\"margin_left\">18</property><property name=\"label\" trans"
  "latable=\"yes\">When laptop lid is closed:</property><property name=\"x"
  "align\">0</property></object><packing><property name=\"left_attach\">0<"
  "/property><property name=\"top_attach\">4</property></packing></child><"
  "child><object class=\"GtkComboBox\" id=\"lid-on-ac-combo\"><property na"
  "me=\"visible\">True</property><property name=\"can_focus\">False</prope"
  "rty><property name=\"hexpand\">True</property><signal name=\"changed\" "
  "handler=\"on_ac_lid_changed_cb\" swapped=\"no\"/><child><object class=\""
  "GtkCellRendererText\" id=\"renderer10\"/><attributes><attribute name=\""
  "text\">0</attribute></attributes></child></object><packing><property na"
  "me=\"left_attach\">1</property><property name=\"top_attach\">4</propert"
  "y></packing></child><child><object class=\"GtkLabel\" id=\"lid-plugged-"
  "in-header\"><property name=\"visible\">True</property><property name=\""
  "can_focus\">False</property><property name=\"margin_top\">12</property>"
  "<property name=\"label\" translatable=\"yes\">&lt;b&gt;Laptop Lid&lt;/b"
  "&gt;</property><property name=\"use_markup\">True</property><property n"
  "ame=\"xalign\">0</property></object><packing><property name=\"left_atta"
  "ch\">0</property><property name=\"top_attach\">3</property><property na"
  "me=\"width\">2</property></packing></child><child><object class=\"GtkLa"
  "bel\" id=\"system-sleep-inactivity-label1\"><property name=\"visible\">"
  "True</property><property name=\"can_focus\">False</property><property n"
  "ame=\"margin_left\">18</property><property name=\"label\" translatable="
  "\"yes\">When inactive for</property><property name=\"xalign\">0</proper"
  "ty></object><packing><property name=\"left_attach\">0</property><proper"
  "ty name=\"top_attach\">2</property></packing></child><child><object cla"
  "ss=\"GtkBox\"><property name=\"visible\">True</property><property name="
  "\"can_focus\">False</property><property name=\"orientation\">vertical</"
  "property><child><object class=\"GtkScale\" id=\"system-sleep-inactivity"
  "-on-ac\"><property name=\"visible\">True</property><property name=\"can"
  "_focus\">True</property><property name=\"adjustment\">system-sleep-inac"
  "tivity-on-ac-adjustment</property><property name=\"round_digits\">0</pr"
  "operty><property name=\"digits\">0</property><property name=\"draw_valu"
  "e\">False</property><signal name=\"value-changed\" handler=\"inactivity"
  "_on_ac_value_changed_cb\" swapped=\"no\"/><style><class name=\"xfce4-sc"
  "ale-label\"/></style></object><packing><property name=\"expand\">False<"
  "/property><property name=\"fill\">True</property><property name=\"posit"
  "ion\">0</property></packing></child><child><object class=\"GtkLabel\" i"
  "d=\"system-sleep-inactivity-on-ac-label\"><property name=\"visible\">Tr"
  "ue</property><property name=\"sensitive\">False</property><property nam"
  "e=\"can_focus\">False</property><property name=\"xalign\">0.5</property"
  "></object><packing><property name=\"expand\">True</property><property n"
  "ame=\"fill\">True</property><property name=\"position\">1</property></p"
  "acking></child></object><packing><property name=\"left_attach\">1</prop"
  "erty><property name=\"top_attach\">2</property></packing></child><child"
  "><object class=\"GtkComboBox\" id=\"system-sleep-mode-on-ac\"><property"
  " name=\"visible\">True</property><property name=\"can_focus\">False</pr"
  "operty><property name=\"hexpand\">True</property><signal name=\"changed"
  "\" handler=\"on_ac_sleep_mode_changed_cb\" swapped=\"no\"/><child><obje"
  "ct class=\"GtkCellRendererText\" id=\"renderer2\"/><attributes><attribu"
  "te name=\"text\">0</attribute></attributes></child></object><packing><p"
  "roperty name=\"left_attach\">1</property><property name=\"top_attach\">"
  "1</property></packing></child><child><object class=\"GtkLabel\" id=\"sy"
  "stem-sleep-mode-label1\"><property name=\"visible\">True</property><pro"
  "perty name=\"can_focus\">False</property><property name=\"margin_left\""
  ">18</property><property name=\"label\" translatable=\"yes\">System slee"
  "p mode:</property><property name=\"xalign\">0</property></object><packi"
  "ng><property name=\"left_attach\">0</property><property name=\"top_atta"
  "ch\">1</property></packing></child><child><object class=\"GtkLabel\" id"
  "=\"label8\"><property name=\"visible\">True</property><property name=\""
  "can_focus\">False</property><property name=\"label\" translatable=\"yes"
  "\">&lt;b&gt;System power saving&lt;/b&gt;</property><property name=\"us"
  "e_markup\">True</property><property name=\"xalign\">0</property></objec"
  "t><packing><property name=\"left_attach\">0</property><property name=\""
  "top_attach\">0</property><property name=\"width\">2</property></packing"
  "></child></object><packing><property name=\"name\">page1</property><pro"
  "perty name=\"title\" translatable=\"yes\">Plugged in</property><propert"
  "y name=\"position\">1</property></packing></child></object><packing><pr"
  "operty name=\"expand\">False</property><property name=\"fill\">True</pr"
  "operty><property name=\"position\">1</property></packing></child><child"
  "><object class=\"GtkFrame\" id=\"critical-power-frame\"><property name="
  "\"visible\">True</property><property name=\"can_focus\">False</property"
  "><property name=\"margin_top\">12</property><property name=\"label_xali"
  "gn\">0</property><property name=\"shadow_type\">none</property><child><"
  "object class=\"GtkBox\" id=\"vbox4\"><property name=\"visible\">True</p"
  "roperty><property name=\"can_focus\">False</property><property name=\"m"
  "argin_start\">18</property><property name=\"margin_top\">6</property><p"
  "roperty name=\"margin_bottom\">12</property><property name=\"orientatio"
  "n\">vertical</property><property name=\"spacing\">3</property><child><o"
  "bject class=\"GtkBox\" id=\"hbox2\"><property name=\"visible\">True</pr"
  "operty><property name=\"can_focus\">False</property><property name=\"sp"
  "acing\">6</property><child><object class=\"GtkLabel\" id=\"critical-pow"
  "er-level-label\"><property name=\"visible\">True</property><property na"
  "me=\"can_focus\">False</property><property name=\"label\" translatable="
  "\"yes\">Critical battery power level:</property><property name=\"xalign"
  "\">0</property></object><packing><property name=\"expand\">False</prope"
  "rty><property name=\"fill\">True</property><property name=\"position\">"
  "0</property></packing></child><child><object class=\"GtkSpinButton\" id"
  "=\"critical-power-level-spin\"><property name=\"width_request\">100</pr"
  "operty><property name=\"visible\">True</property><property name=\"can_f"
  "ocus\">True</property><property name=\"invisible_char\">\342\227\217</p"
  "roperty><property name=\"width_chars\">6</property><property name=\"pri"
  "mary_icon_activatable\">False</property><property name=\"secondary_icon"
  "_activatable\">False</property><property name=\"adjustment\">critical-p"
  "ower-level-spin-adjustment</property><signal name=\"value-changed\" han"
  "dler=\"critical_level_value_changed_cb\" swapped=\"no\"/></object><pack"
  "ing><property name=\"expand\">False</property><property name=\"fill\">T"
  "rue</property><property name=\"position\">1</property></packing></child"
  "><child><object class=\"GtkLabel\"><property name=\"visible\">True</pro"
  "perty><property name=\"can_focus\">False</property><property name=\"lab"
  "el\" translatable=\"yes\">%</property></object><packing><property name="
  "\"expand\">False</property><property name=\"fill\">True</property><prop"
  "erty name=\"position\">2</property></packing></child></object><packing>"
  "<property name=\"expand\">False</property><property name=\"fill\">True<"
  "/property><property name=\"position\">0</property></packing></child><ch"
  "ild><object class=\"GtkBox\" id=\"hbox3\"><property name=\"visible\">Tr"
  "ue</property><property name=\"can_focus\">False</property><property nam"
  "e=\"spacing\">6</property><child><object class=\"GtkLabel\" id=\"critic"
  "al-power-action-label\"><property name=\"visible\">True</property><prop"
  "erty name=\"can_focus\">False</property><property name=\"label\" transl"
  "atable=\"yes\">On critical battery power:</property><property name=\"xa"
  "lign\">0</property></object><packing><property name=\"expand\">False</p"
  "roperty><property name=\"fill\">True</property><property name=\"positio"
  "n\">0</property></packing></child><child><object class=\"GtkComboBox\" "
  "id=\"critical-power-action-combo\"><property name=\"visible\">True</pro"
  "perty><property name=\"can_focus\">False</property><signal name=\"chang"
  "ed\" handler=\"battery_critical_changed_cb\" swapped=\"no\"/><child><ob"
  "ject class=\"GtkCellRendererText\" id=\"renderer7\"/><attributes><attri"
  "bute name=\"text\">0</attribute></attributes></child></object><packing>"
  "<property name=\"expand\">True</property><property name=\"fill\">True</"
  "property><property name=\"position\">1</property></packing></child></ob"
  "ject><packing><property name=\"expand\">False</property><property name="
  "\"fill\">True</property><property name=\"position\">1</property></packi"
  "ng></child></object></child><child type=\"label\"><object class=\"GtkLa"
  "bel\" id=\"label9\"><property name=\"visible\">True</property><property"
  " name=\"can_focus\">False</property><property name=\"label\" translatab"
  "le=\"yes\">&lt;b&gt;Critical power&lt;/b&gt;</property><property name=\""
  "use_markup\">True</property></object></child></object><packing><propert"
  "y name=\"expand\">False</property><property name=\"fill\">True</propert"
  "y><property name=\"position\">2</property></packing></child><child><obj"
  "ect class=\"GtkFrame\" id=\"security-frame\"><property name=\"visible\""
  ">True</property><property name=\"can_focus\">False</property><property "
  "name=\"margin_top\">6</property><property name=\"label_xalign\">0</prop"
  "erty><property name=\"shadow_type\">none</property><child><object class"
  "=\"GtkBox\" id=\"vbox11\"><property name=\"visible\">True</property><pr"
  "operty name=\"can_focus\">False</property><property name=\"margin_start"
  "\">6</property><property name=\"margin_top\">6</property><property name"
  "=\"margin_bottom\">12</property><property name=\"orientation\">vertical"
  "</property><property name=\"spacing\">6</property><child><object class="
  "\"GtkCheckButton\" id=\"lock-screen\"><property name=\"label\" translat"
  "able=\"yes\">Lock screen when system is going to sleep</property><prope"
  "rty name=\"visible\">True</property><property name=\"can_focus\">True</"
  "property><property name=\"receives_default\">False</property><property "
  "name=\"margin_left\">6</property><property name=\"draw_indicator\">True"
  "</property><signal name=\"toggled\" handler=\"lock_screen_toggled_cb\" "
  "swapped=\"no\"/></object><packing><property name=\"expand\">False</prop"
  "erty><property name=\"fill\">True</property><property name=\"position\""
  ">0</property></packing></child></object></child><child type=\"label\"><"
  "object class=\"GtkLabel\" id=\"label5\"><property name=\"visible\">True"
  "</property><property name=\"can_focus\">False</property><property name="
  "\"label\" translatable=\"yes\">&lt;b&gt;Security&lt;/b&gt;</property><p"
  "roperty name=\"use_markup\">True</property></object></child></object><p"
  "acking><property name=\"expand\">False</property><property name=\"fill\""
  ">True</property><property name=\"position\">3</property></packing></chi"
  "ld></object><packing><property name=\"position\">1</property></packing>"
  "</child><child type=\"tab\"><object class=\"GtkLabel\" id=\"system\"><p"
  "roperty name=\"visible\">True</property><property name=\"can_focus\">Fa"
  "lse</property><property name=\"label\" translatable=\"yes\">System</pro"
  "perty></object><packing><property name=\"position\">1</property><proper"
  "ty name=\"tab_fill\">False</property></packing></child><child><object c"
  "lass=\"GtkBox\" id=\"display-tab\"><property name=\"visible\">True</pro"
  "perty><property name=\"can_focus\">False</property><property name=\"val"
  "ign\">start</property><property name=\"vexpand\">True</property><proper"
  "ty name=\"border_width\">12</property><property name=\"orientation\">ve"
  "rtical</property><property name=\"spacing\">6</property><child><object "
  "class=\"GtkStackSwitcher\" id=\"display-stack-switcher\"><property name"
  "=\"visible\">True</property><property name=\"can_focus\">False</propert"
  "y><property name=\"halign\">center</property><property name=\"valign\">"
  "center</property><property name=\"margin_bottom\">6</property><property"
  " name=\"stack\">display-stack</property></object><packing><property nam"
  "e=\"expand\">True</property><property name=\"fill\">True</property><pro"
  "perty name=\"position\">0</property></packing></child><child><object cl"
  "ass=\"GtkBox\" id=\"dpms-frame\"><property name=\"visible\">True</prope"
  "rty><property name=\"can_focus\">False</property><child><object class=\""
  "GtkLabel\" id=\"dpms-label\"><property name=\"visible\">True</property>"
  "<property name=\"can_focus\">False</property><property name=\"label\" t"
  "ranslatable=\"yes\">&lt;b&gt;Display power management&lt;/b&gt;</proper"
  "ty><property name=\"use_markup\">True</property></object><packing><prop"
  "erty name=\"expand\">False</property><property name=\"fill\">True</prop"
  "erty><property name=\"position\">0</property></packing></child><child><"
  "object class=\"GtkSwitch\" id=\"handle-dpms\"><property name=\"visible\""
  ">True</property><property name=\"can_focus\">True</property><property n"
  "ame=\"tooltip_text\" translatable=\"yes\">Let the power manager handle "
  "display power management (DPMS) instead of X11.</property><signal name="
  "\"state-set\" handler=\"dpms_toggled_cb\" swapped=\"no\"/></object><pac"
  "king><property name=\"expand\">False</property><property name=\"fill\">"
  "True</property><property name=\"pack_type\">end</property><property nam"
  "e=\"position\">1</property></packing></child></object><packing><propert"
  "y name=\"expand\">False</property><property name=\"fill\">True</propert"
  "y><property name=\"position\">1</property></packing></child><child><obj"
  "ect class=\"GtkStack\" id=\"display-stack\"><property name=\"visible\">"
  "True</property><property name=\"can_focus\">False</property><property n"
  "ame=\"transition_type\">slide-left-right</property><child><object class"
  "=\"GtkGrid\" id=\"display-battery-grid\"><property name=\"visible\">Tru"
  "e</property><property name=\"can_focus\">False</property><property name"
  "=\"row_spacing\">6</property><property name=\"column_spacing\">12</prop"
  "erty><child><object class=\"GtkLabel\" id=\"display-blank-label\"><prop"
  "erty name=\"visible\">True</property><property name=\"can_focus\">False"
  "</property><property name=\"margin_left\">18</property><property name=\""
  "label\" translatable=\"yes\">Blank after</property><property name=\"xal"
  "ign\">0</property></object><packing><property name=\"left_attach\">0</p"
  "roperty><property name=\"top_attach\">0</property></packing></child><ch"
  "ild><object class=\"GtkBox\"><property name=\"visible\">True</property>"
  "<property name=\"can_focus\">False</property><property name=\"orientati"
  "on\">vertical</property><child><object class=\"GtkScale\" id=\"display-"
  "blank-on-battery\"><property name=\"visible\">True</property><property "
  "name=\"can_focus\">True</property><property name=\"hexpand\">True</prop"
  "erty><property name=\"adjustment\">display-blank-on-battery-adjustment<"
  "/property><property name=\"round_digits\">0</property><property name=\""
  "digits\">0</property><property name=\"draw_value\">False</property><sig"
  "nal name=\"value-changed\" handler=\"display_blank_on_battery_value_cha"
  "nged_cb\" swapped=\"no\"/><style><class name=\"xfce4-scale-label\"/></s"
  "tyle></object><packing><property name=\"expand\">False</property><prope"
  "rty name=\"fill\">True</property><property name=\"position\">0</propert"
  "y></packing></child><child><object class=\"GtkLabel\" id=\"display-blan"
  "k-on-battery-label\"><property name=\"visible\">True</property><propert"
  "y name=\"sensitive\">False</property><property name=\"can_focus\">False"
  "</property><property name=\"xalign\">0.5</property></object><packing><p"
  "roperty name=\"expand\">False</property><property name=\"fill\">True</p"
  "roperty><property name=\"position\">1</property></packing></child></obj"
  "ect><packing><property name=\"left_attach\">1</property><property name="
  "\"top_attach\">0</property></packing></child><child><object class=\"Gtk"
  "Box\"><property name=\"visible\">True</property><property name=\"can_fo"
  "cus\">False</property><property name=\"orientation\">vertical</property"
  "><child><object class=\"GtkScale\" id=\"dpms-sleep-on-battery\"><proper"
  "ty name=\"visible\">True</property><property name=\"can_focus\">True</p"
  "roperty><property name=\"hexpand\">True</property><property name=\"adju"
  "stment\">dpms-sleep-on-battery-adjustment</property><property name=\"ro"
  "und_digits\">0</property><property name=\"digits\">0</property><propert"
  "y name=\"draw_value\">False</property><signal name=\"value-changed\" ha"
  "ndler=\"sleep_on_battery_value_changed_cb\" swapped=\"no\"/><style><cla"
  "ss name=\"xfce4-scale-label\"/></style></object><packing><property name"
  "=\"expand\">False</property><property name=\"fill\">True</property><pro"
  "perty name=\"position\">0</property></packing></child><child><object cl"
  "ass=\"GtkLabel\" id=\"dpms-sleep-on-battery-label\"><property name=\"vi"
  "sible\">True</property><property name=\"sensitive\">False</property><pr"
  "operty name=\"can_focus\">False</property><property name=\"xalign\">0.5"
  "</property></object><packing><property name=\"expand\">False</property>"
  "<property name=\"fill\">True</property><property name=\"position\">1</p"
  "roperty></packing></child></object><packing><property name=\"left_attac"
  "h\">1</property><property name=\"top_attach\">1</property></packing></c"
  "hild><child><object class=\"GtkLabel\" id=\"dpms-sleep-label\"><propert"
  "y name=\"visible\">True</property><property name=\"can_focus\">False</p"
  "roperty><property name=\"margin_left\">18</property><property name=\"la"
  "bel\" translatable=\"yes\">Put to sleep after</property><property name="
  "\"xalign\">0</property></object><packing><property name=\"left_attach\""
  ">0</property><property name=\"top_attach\">1</property></packing></chil"
  "d><child><object class=\"GtkLabel\" id=\"dpms-off-label\"><property nam"
  "e=\"visible\">True</property><property name=\"can_focus\">False</proper"
  "ty><property name=\"margin_left\">18</property><property name=\"label\""
  " translatable=\"yes\">Switch off after</property><property name=\"xalig"
  "n\">0</property></object><packing><property name=\"left_attach\">0</pro"
  "perty><property name=\"top_attach\">2</property></packing></child><chil"
  "d><object class=\"GtkBox\"><property name=\"visible\">True</property><p"
  "roperty name=\"can_focus\">False</property><property name=\"hexpand\">T"
  "rue</property><property name=\"orientation\">vertical</property><child>"
  "<object class=\"GtkScale\" id=\"dpms-off-on-battery\"><property name=\""
  "visible\">True</property><property name=\"can_focus\">True</property><p"
  "roperty name=\"hexpand\">True</property><property name=\"adjustment\">d"
  "pms-off-on-battery-adjustment</property><property name=\"round_digits\""
  ">0</property><property name=\"digits\">0</property><property name=\"dra"
  "w_value\">False</property><signal name=\"value-changed\" handler=\"off_"
  "on_battery_value_changed_cb\" swapped=\"no\"/><style><class name=\"xfce"
  "4-scale-label\"/></style></object><packing><property name=\"expand\">Fa"
  "lse</property><property name=\"fill\">True</property><property name=\"p"
  "osition\">0</property></packing></child><child><object class=\"GtkLabel"
  "\" id=\"dpms-off-on-battery-label\"><property name=\"visible\">True</pr"
  "operty><property name=\"sensitive\">False</property><property name=\"ca"
  "n_focus\">False</property><property name=\"xalign\">0.5</property></obj"
  "ect><packing><property name=\"expand\">False</property><property name=\""
  "fill\">True</property><property name=\"position\">1</property></packing"
  "></child></object><packing><property name=\"left_attach\">1</property><"
  "property name=\"top_attach\">2</property></packing></child><child><obje"
  "ct class=\"GtkLabel\" id=\"brightness-level-label\"><property name=\"vi"
  "sible\">True</property><property name=\"can_focus\">False</property><pr"
  "operty name=\"margin_left\">18</property><property name=\"label\" trans"
  "latable=\"yes\">On inactivity reduce to</property><property name=\"xali"
  "gn\">0</property></object><packing><property name=\"left_attach\">0</pr"
  "operty><property name=\"top_attach\">4</property></packing></child><chi"
  "ld><object class=\"GtkLabel\" id=\"brightness-inactivity-label\"><prope"
  "rty name=\"visible\">True</property><property name=\"can_focus\">False<"
  "/property><property name=\"margin_left\">18</property><property name=\""
  "label\" translatable=\"yes\">Reduce after</property><property name=\"xa"
  "lign\">0</property></object><packing><property name=\"left_attach\">0</"
  "property><property name=\"top_attach\">5</property></packing></child><c"
  "hild><object class=\"GtkBox\"><property name=\"visible\">True</property"
  "><property name=\"can_focus\">False</property><property name=\"orientat"
  "ion\">vertical</property><child><object class=\"GtkScale\" id=\"brightn"
  "ess-level-on-battery\"><property name=\"visible\">True</property><prope"
  "rty name=\"can_focus\">True</property><property name=\"hexpand\">True</"
  "property><property name=\"adjustment\">brightness-level-on-battery-adju"
  "stment</property><property name=\"round_digits\">0</property><property "
  "name=\"digits\">0</property><property name=\"draw_value\">False</proper"
  "ty><signal name=\"value-changed\" handler=\"brightness_level_on_battery"
  "\" swapped=\"no\"/><style><class name=\"xfce4-scale-label\"/></style></"
  "object><packing><property name=\"expand\">False</property><property nam"
  "e=\"fill\">True</property><property name=\"position\">0</property></pac"
  "king></child><child><object class=\"GtkLabel\" id=\"brightness-level-on"
  "-battery-label\"><property name=\"visible\">True</property><property na"
  "me=\"sensitive\">False</property><property name=\"can_focus\">False</pr"
  "operty><property name=\"xalign\">0.5</property></object><packing><prope"
  "rty name=\"expand\">False</property><property name=\"fill\">True</prope"
  "rty><property name=\"position\">1</property></packing></child></object>"
  "<packing><property name=\"left_attach\">1</property><property name=\"to"
  "p_attach\">4</property></packing></child><child><object class=\"GtkBox\""
  "><property name=\"visible\">True</property><property name=\"can_focus\""
  ">False</property><property name=\"orientation\">vertical</property><chi"
  "ld><object class=\"GtkScale\" id=\"brightness-inactivity-on-battery\"><"
  "property name=\"visible\">True</property><property name=\"can_focus\">T"
  "rue</property><property name=\"hexpand\">True</property><property name="
  "\"adjustment\">brightness-inactivity-on-battery-adjustment</property><p"
  "roperty name=\"round_digits\">0</property><property name=\"digits\">0</"
  "property><property name=\"draw_value\">False</property><signal name=\"v"
  "alue-changed\" handler=\"brightness_on_battery_value_changed_cb\" swapp"
  "ed=\"no\"/><style><class name=\"xfce4-scale-label\"/></style></object><"
  "packing><property name=\"expand\">False</property><property name=\"fill"
  "\">True</property><property name=\"position\">0</property></packing></c"
  "hild><child><object class=\"GtkLabel\" id=\"brightness-inactivity-on-ba"
  "ttery-label\"><property name=\"visible\">True</property><property name="
  "\"sensitive\">False</property><property name=\"can_focus\">False</prope"
  "rty><property name=\"xalign\">0.5</property></object><packing><property"
  " name=\"expand\">False</property><property name=\"fill\">True</property"
  "><property name=\"position\">1</property></packing></child></object><pa"
  "cking><property name=\"left_attach\">1</property><property name=\"top_a"
  "ttach\">5</property></packing></child><child><object class=\"GtkLabel\""
  " id=\"label4\"><property name=\"visible\">True</property><property name"
  "=\"can_focus\">False</property><property name=\"margin_top\">12</proper"
  "ty><property name=\"label\" translatable=\"yes\">&lt;b&gt;Brightness re"
  "duction&lt;/b&gt;</property><property name=\"use_markup\">True</propert"
  "y><property name=\"xalign\">0</property></object><packing><property nam"
  "e=\"left_attach\">0</property><property name=\"top_attach\">3</property"
  "><property name=\"width\">2</property></packing></child></object><packi"
  "ng><property name=\"name\">page0</property><property name=\"title\" tra"
  "nslatable=\"yes\">On battery</property></packing></child><child><object"
  " class=\"GtkGrid\" id=\"display-pluggedin-grid\"><property name=\"visib"
  "le\">True</property><property name=\"can_focus\">False</property><prope"
  "rty name=\"row_spacing\">6</property><property name=\"column_spacing\">"
  "12</property><child><object class=\"GtkBox\"><property name=\"visible\""
  ">True</property><property name=\"can_focus\">False</property><property "
  "name=\"orientation\">vertical</property><child><object class=\"GtkScale"
  "\" id=\"display-blank-on-ac\"><property name=\"visible\">True</property"
  "><property name=\"can_focus\">True</property><property name=\"hexpand\""
  ">True</property><property name=\"adjustment\">display-blank-on-ac-adjus"
  "tment</property><property name=\"round_digits\">0</property><property n"
  "ame=\"digits\">0</property><property name=\"draw_value\">False</propert"
  "y><signal name=\"value-changed\" handler=\"display_blank_on_ac_value_ch"
  "anged_cb\" swapped=\"no\"/><style><class name=\"xfce4-scale-label\"/></"
  "style></object><packing><property name=\"expand\">False</property><prop"
  "erty name=\"fill\">True</property><property name=\"position\">0</proper"
  "ty></packing></child><child><object class=\"GtkLabel\" id=\"display-bla"
  "nk-on-ac-label\"><property name=\"visible\">True</property><property na"
  "me=\"sensitive\">False</property><property name=\"can_focus\">False</pr"
  "operty><property name=\"xalign\">0.5</property></object><packing><prope"
  "rty name=\"expand\">False</property><property name=\"fill\">True</prope"
  "rty><property name=\"position\">1</property></packing></child></object>"
  "<packing><property name=\"left_attach\">1</property><property name=\"to"
  "p_attach\">0</property></packing></child><child><object class=\"GtkLabe"
  "l\" id=\"display-blank-label1\"><property name=\"visible\">True</proper"
  "ty><property name=\"can_focus\">False</property><property name=\"margin"
  "_left\">18</property><property name=\"label\" translatable=\"yes\">Blan"
  "k after</property><property name=\"xalign\">0</property></object><packi"
  "ng><property name=\"left_attach\">0</property><property name=\"top_atta"
  "ch\">0</property></packing></child><child><object class=\"GtkLabel\" id"
  "=\"dpms-sleep-label1\"><property name=\"visible\">True</property><prope"
  "rty name=\"can_focus\">False</property><property name=\"margin_left\">1"
  "8</property><property name=\"label\" translatable=\"yes\">Put to sleep "
  "after</property><property name=\"xalign\">0</property></object><packing"
  "><property name=\"left_attach\">0</property><property name=\"top_attach"
  "\">1</property></packing></child><child><object class=\"GtkLabel\" id=\""
  "dpms-off-label1\"><property name=\"visible\">True</property><property n"
  "ame=\"can_focus\">False</property><property name=\"margin_left\">18</pr"
  "operty><property name=\"label\" translatable=\"yes\">Switch off after</"
  "property><property name=\"xalign\">0</property></object><packing><prope"
  "rty name=\"left_attach\">0</property><property name=\"top_attach\">2</p"
  "roperty></packing></child><child><object class=\"GtkLabel\" id=\"label6"
  "\"><property name=\"visible\">True</property><property name=\"can_focus"
  "\">False</property><property name=\"margin_top\">12</property><property"
  " name=\"label\" translatable=\"yes\">&lt;b&gt;Brightness reduction&lt;/"
  "b&gt;</property><property name=\"use_markup\">True</property><property "
  "name=\"xalign\">0</property></object><packing><property name=\"left_att"
  "ach\">0</property><property name=\"top_attach\">3</property><property n"
  "ame=\"width\">2</property></packing></child><child><object class=\"GtkL"
  "abel\" id=\"brightness-level-label1\"><property name=\"visible\">True</"
  "property><property name=\"can_focus\">False</property><property name=\""
  "margin_left\">18</property><property name=\"margin_top\">6</property><p"
  "roperty name=\"label\" translatable=\"yes\">On inactivity reduce to</pr"
  "operty><property name=\"xalign\">0</property></object><packing><propert"
  "y name=\"left_attach\">0</property><property name=\"top_attach\">4</pro"
  "perty></packing></child><child><object class=\"GtkLabel\" id=\"brightne"
  "ss-inactivity-label1\"><property name=\"visible\">True</property><prope"
  "rty name=\"can_focus\">False</property><property name=\"margin_left\">1"
  "8</property><property name=\"label\" translatable=\"yes\">Reduce after<"
  "/property><property name=\"xalign\">0</property></object><packing><prop"
  "erty name=\"left_attach\">0</property><property name=\"top_attach\">5</"
  "property></packing></child><child><object class=\"GtkBox\"><property na"
  "me=\"visible\">True</property><property name=\"can_focus\">False</prope"
  "rty><property name=\"orientation\">vertical</property><child><object cl"
  "ass=\"GtkScale\" id=\"dpms-sleep-on-ac\"><property name=\"visible\">Tru"
  "e</property><property name=\"can_focus\">True</property><property name="
  "\"hexpand\">True</property><property name=\"adjustment\">dpms-sleep-on-"
  "ac-adjustment</property><property name=\"round_digits\">0</property><pr"
  "operty name=\"digits\">0</property><property name=\"draw_value\">False<"
  "/property><signal name=\"value-changed\" handler=\"sleep_on_ac_value_ch"
  "anged_cb\" swapped=\"no\"/><style><class name=\"xfce4-scale-label\"/></"
  "style></object><packing><property name=\"expand\">False</property><prop"
  "erty name=\"fill\">True</property><property name=\"position\">0</proper"
  "ty></packing></child><child><object class=\"GtkLabel\" id=\"dpms-sleep-"
  "on-ac-label\"><property name=\"visible\">True</property><property name="
  "\"sensitive\">False</property><property name=\"can_focus\">False</prope"
  "rty><property name=\"xalign\">0.5</property></object><packing><property"
  " name=\"expand\">False</property><property name=\"fill\">True</property"
  "><property name=\"position\">1</property></packing></child></object><pa"
  "cking><property name=\"left_attach\">1</property><property name=\"top_a"
  "ttach\">1</property></packing></child><child><object class=\"GtkBox\"><"
  "property name=\"visible\">True</property><property name=\"can_focus\">F"
  "alse</property><property name=\"orientation\">vertical</property><child"
  "><object class=\"GtkScale\" id=\"dpms-off-on-ac\"><property name=\"visi"
  "ble\">True</property><property name=\"can_focus\">True</property><prope"
  "rty name=\"hexpand\">True</property><property name=\"adjustment\">dpms-"
  "off-on-ac-adjustment</property><property name=\"round_digits\">0</prope"
  "rty><property name=\"digits\">0</property><property name=\"draw_value\""
  ">False</property><signal name=\"value-changed\" handler=\"off_on_ac_val"
  "ue_changed_cb\" swapped=\"no\"/><style><class name=\"xfce4-scale-label\""
  "/></style></object><packing><property name=\"expand\">False</property><"
  "property name=\"fill\">True</property><property name=\"position\">0</pr"
  "operty></packing></child><child><object class=\"GtkLabel\" id=\"dpms-of"
  "f-on-ac-label\"><property name=\"visible\">True</property><property nam"
  "e=\"sensitive\">False</property><property name=\"can_focus\">False</pro"
  "perty><property name=\"xalign\">0.5</property></object><packing><proper"
  "ty name=\"expand\">False</property><property name=\"fill\">True</proper"
  "ty><property name=\"position\">1</property></packing></child></object><"
  "packing><property name=\"left_attach\">1</property><property name=\"top"
  "_attach\">2</property></packing></child><child><object class=\"GtkBox\""
  "><property name=\"visible\">True</property><property name=\"can_focus\""
  ">False</property><property name=\"margin_top\">6</property><property na"
  "me=\"orientation\">vertical</property><child><object class=\"GtkScale\""
  " id=\"brightness-level-on-ac\"><property name=\"visible\">True</propert"
  "y><property name=\"can_focus\">True</property><property name=\"hexpand\""
  ">True</property><property name=\"adjustment\">brightness-level-on-ac-ad"
  "justment</property><property name=\"round_digits\">0</property><propert"
  "y name=\"digits\">0</property><property name=\"draw_value\">False</prop"
  "erty><signal name=\"value-changed\" handler=\"brightness_level_on_ac\" "
  "swapped=\"no\"/><style><class name=\"xfce4-scale-label\"/></style></obj"
  "ect><packing><property name=\"expand\">False</property><property name=\""
  "fill\">True</property><property name=\"position\">0</property></packing"
  "></child><child><object class=\"GtkLabel\" id=\"brightness-level-on-ac-"
  "label\"><property name=\"visible\">True</property><property name=\"sens"
  "itive\">False</property><property name=\"can_focus\">False</property><p"
  "roperty name=\"xalign\">0.5</property></object><packing><property name="
  "\"expand\">False</property><property name=\"fill\">True</property><prop"
  "erty name=\"position\">1</property></packing></child></object><packing>"
  "<property name=\"left_attach\">1</property><property name=\"top_attach\""
  ">4</property></packing></child><child><object class=\"GtkBox\"><propert"
  "y name=\"visible\">True</property><property name=\"can_focus\">False</p"
  "roperty><property name=\"hexpand\">True</property><property name=\"orie"
  "ntation\">vertical</property><child><object class=\"GtkScale\" id=\"bri"
  "ghtness-inactivity-on-ac\"><property name=\"visible\">True</property><p"
  "roperty name=\"can_focus\">True</property><property name=\"adjustment\""
  ">brightness-inactivity-on-ac-adjustment</property><property name=\"roun"
  "d_digits\">0</property><property name=\"digits\">0</property><property "
  "name=\"draw_value\">False</property><signal name=\"value-changed\" hand"
  "ler=\"brightness_on_ac_value_changed_cb\" swapped=\"no\"/><style><class"
  " name=\"xfce4-scale-label\"/></style></object><packing><property name=\""
  "expand\">False</property><property name=\"fill\">True</property><proper"
  "ty name=\"position\">0</property></packing></child><child><object class"
  "=\"GtkLabel\" id=\"brightness-inactivity-on-ac-label\"><property name=\""
  "visible\">True</property><property name=\"sensitive\">False</property><"
  "property name=\"can_focus\">False</property><property name=\"xalign\">0"
  ".5</property></object><packing><property name=\"expand\">False</propert"
  "y><property name=\"fill\">True</property><property name=\"position\">1<"
  "/property></packing></child></object><packing><property name=\"left_att"
  "ach\">1</property><property name=\"top_attach\">5</property></packing><"
  "/child></object><packing><property name=\"name\">page1</property><prope"
  "rty name=\"title\" translatable=\"yes\">Plugged in</property><property "
  "name=\"position\">1</property></packing></child></object><packing><prop"
  "erty name=\"expand\">True</property><property name=\"fill\">True</prope"
  "rty><property name=\"position\">3</property></packing></child></object>"
  "<packing><property name=\"position\">2</property></packing></child><chi"
  "ld type=\"tab\"><object class=\"GtkLabel\" id=\"display\"><property nam"
  "e=\"visible\">True</property><property name=\"can_focus\">False</proper"
  "ty><property name=\"label\" translatable=\"yes\">Display</property></ob"
  "ject><packing><property name=\"position\">2</property><property name=\""
  "tab_fill\">False</property></packing></child><child><object class=\"Gtk"
  "Box\" id=\"light-locker-vbox1\"><property name=\"visible\">True</proper"
  "ty><property name=\"can_focus\">False</property><property name=\"border"
  "_width\">12</property><property name=\"orientation\">vertical</property"
  "><property name=\"spacing\">6</property><child><object class=\"GtkFrame"
  "\" id=\"light-locker-frame\"><property name=\"visible\">True</property>"
  "<property name=\"can_focus\">False</property><property name=\"label_xal"
  "ign\">0</property><property name=\"shadow_type\">none</property><child>"
  "<object class=\"GtkBox\" id=\"light-locker-vbox2\"><property name=\"vis"
  "ible\">True</property><property name=\"can_focus\">False</property><pro"
  "perty name=\"margin_start\">18</property><property name=\"margin_top\">"
  "6</property><property name=\"margin_bottom\">18</property><property nam"
  "e=\"orientation\">vertical</property><property name=\"spacing\">5</prop"
  "erty><child><object class=\"GtkGrid\" id=\"light-locker-table\"><proper"
  "ty name=\"visible\">True</property><property name=\"can_focus\">False</"
  "property><property name=\"row_spacing\">6</property><property name=\"co"
  "lumn_spacing\">6</property><child><object class=\"GtkLabel\" id=\"light"
  "-locker-automatic-locking-label\"><property name=\"visible\">True</prop"
  "erty><property name=\"can_focus\">False</property><property name=\"labe"
  "l\" translatable=\"yes\">Automatically lock the session:</property><pro"
  "perty name=\"xalign\">0</property></object><packing><property name=\"le"
  "ft_attach\">0</property><property name=\"top_attach\">0</property></pac"
  "king></child><child><object class=\"GtkComboBox\" id=\"light-locker-aut"
  "omatic-locking-combo\"><property name=\"visible\">True</property><prope"
  "rty name=\"can_focus\">False</property><property name=\"model\">liststo"
  "re1</property><property name=\"active\">0</property><signal name=\"chan"
  "ged\" handler=\"light_locker_automatic_locking_changed_cb\" swapped=\"n"
  "o\"/><child><object class=\"GtkCellRendererText\" id=\"renderer11\"/><a"
  "ttributes><attribute name=\"text\">1</attribute></attributes></child></"
  "object><packing><property name=\"left_attach\">1</property><property na"
  "me=\"top_attach\">0</property></packing></child><child><object class=\""
  "GtkLabel\" id=\"light-locker-late-locking-label\"><property name=\"visi"
  "ble\">True</property><property name=\"can_focus\">False</property><prop"
  "erty name=\"label\" translatable=\"yes\">Delay locking after screensave"
  "r for</property><property name=\"xalign\">0</property></object><packing"
  "><property name=\"left_attach\">0</property><property name=\"top_attach"
  "\">1</property></packing></child><child><object class=\"GtkBox\"><prope"
  "rty name=\"visible\">True</property><property name=\"can_focus\">False<"
  "/property><property name=\"orientation\">vertical</property><child><obj"
  "ect class=\"GtkScale\" id=\"light-locker-late-locking-scale\"><property"
  " name=\"visible\">True</property><property name=\"can_focus\">True</pro"
  "perty><property name=\"adjustment\">light-locker-late-locking-adjustmen"
  "t</property><property name=\"round_digits\">0</property><property name="
  "\"digits\">0</property><property name=\"draw_value\">False</property><s"
  "ignal name=\"value-changed\" handler=\"light_locker_late_locking_value_"
  "changed_cb\" swapped=\"no\"/><style><class name=\"xfce4-scale-label\"/>"
  "</style></object><packing><property name=\"expand\">False</property><pr"
  "operty name=\"fill\">True</property><property name=\"position\">0</prop"
  "erty></packing></child><child><object class=\"GtkLabel\" id=\"light-loc"
  "ker-late-locking-scale-label\"><property name=\"visible\">True</propert"
  "y><property name=\"sensitive\">False</property><property name=\"can_foc"
  "us\">False</property><property name=\"xalign\">0.5</property></object><"
  "packing><property name=\"expand\">False</property><property name=\"fill"
  "\">True</property><property name=\"position\">1</property></packing></c"
  "hild></object><packing><property name=\"left_attach\">1</property><prop"
  "erty name=\"top_attach\">1</property></packing></child><child><object c"
  "lass=\"GtkCheckButton\" id=\"light-locker-suspend\"><property name=\"la"
  "bel\" translatable=\"yes\">Lock screen when system is going to sleep</p"
  "roperty><property name=\"visible\">True</property><property name=\"can_"
  "focus\">True</property><property name=\"receives_default\">False</prope"
  "rty><property name=\"draw_indicator\">True</property><signal name=\"tog"
  "gled\" handler=\"lock_screen_toggled_cb\" swapped=\"no\"/></object><pac"
  "king><property name=\"left_attach\">0</property><property name=\"top_at"
  "tach\">2</property></packing></child><child><placeholder/></child></obj"
  "ect><packing><property name=\"expand\">False</property><property name=\""
  "fill\">True</property><property name=\"position\">0</property></packing"
  "></child></object></child><child type=\"label\"><object class=\"GtkLabe"
  "l\" id=\"light-locker-frame-label\"><property name=\"visible\">True</pr"
  "operty><property name=\"can_focus\">False</property><property name=\"la"
  "bel\" translatable=\"yes\">&lt;b&gt;Light Locker&lt;/b&gt;</property><p"
  "roperty name=\"use_markup\">True</property></object></child></object><p"
  "acking><property name=\"expand\">False</property><property name=\"fill\""
  ">True</property><property name=\"position\">0</property></packing></chi"
  "ld></object><packing><property name=\"position\">3</property></packing>"
  "</child><child type=\"tab\"><object class=\"GtkLabel\" id=\"light-locke"
  "r-tab-label\"><property name=\"visible\">True</property><property name="
  "\"can_focus\">False</property><property name=\"label\" translatable=\"y"
  "es\">Security</property></object><packing><property name=\"position\">2"
  "</property><property name=\"tab_fill\">False</property></packing></chil"
  "d></object><packing><property name=\"expand\">True</property><property "
  "name=\"fill\">True</property><property name=\"position\">0</property></"
  "packing></child></object><packing><property name=\"expand\">True</prope"
  "rty><property name=\"fill\">True</property><property name=\"position\">"
  "1</property></packing></child></object></child><action-widgets><action-"
  "widget response=\"-11\">help-button</action-widget><action-widget respo"
  "nse=\"-3\">close-button</action-widget></action-widgets></object><objec"
  "t class=\"GtkSizeGroup\" id=\"sg-critical-power\"><widgets><widget name"
  "=\"critical-power-level-label\"/><widget name=\"critical-power-action-l"
  "abel\"/></widgets></object></interface>"
};

static const unsigned xfpm_settings_ui_length = 64745u;

